﻿using System.Numerics;
using ProjectAircraftCarrier.Exceptions;
namespace ProjectAircraftCarrier.Generics
{   
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="warship">Добавляемый корабль</param>
        /// <returns></returns>
        public void Insert(T warship, IEqualityComparer<T>? equal = null)
        {
            if (_places.Count == _maxCount)
                throw new StorageOverflowException(_maxCount);
            Insert(warship, 0, equal);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="warship">Добавляемый корабль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public void Insert(T warship, int position, IEqualityComparer<T?>?
            equal = null)
        {
            if (_places.Count == _maxCount)
                throw new StorageOverflowException(_maxCount);
            if (!(position >= 0 && position <= Count))
                throw new Exception("Incorrect insertion position");
            if (equal != null)
            {
                if (_places.Contains(warship, equal))
                    throw new ArgumentException(nameof(warship));
            }
            _places.Insert(position, warship);
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public void Remove(int position)
        {
            if (!(position >= 0 && position < Count))
                throw new WarshipNotFoundException(position);
            _places.RemoveAt(position);
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            get
            {
                if (!(position >= 0 && position < Count))
                    return null;
                return _places[position];
            }
            set
            {
                if (!(position >= 0 && position < Count))
                    return;
                _places.Insert(position, value);
            }
        }
        /// <summary>
        /// Проход по списку
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T?> GetWarships(int? maxWarships = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxWarships.HasValue && i == maxWarships.Value)
                    yield break;
            }
        }
        /// <summary>
        /// Сортировка набора объектов
        /// </summary>
        /// <param name="comparer"></param>
        public void SortSet(IComparer<T?> comparer) => _places.Sort(comparer);
    }
}