﻿using ProjectAircraftCarrier.DrawingObjects;
using ProjectAircraftCarrier.MovementStrategy;
using System.Text;
namespace ProjectAircraftCarrier.Generics
{
    /// <summary>
    /// Класс для хранения коллекции
    /// </summary>
    internal class WarshipsGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<WarshipsCollectionInfo, WarshipsGenericCollection
            <DrawingWarship, DrawingObjectWarship>> _warshipStorages;
        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<WarshipsCollectionInfo> Keys => 
            _warshipStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public WarshipsGenericStorage(int pictureWidth, int pictureHeight)
        {
            _warshipStorages = new Dictionary<WarshipsCollectionInfo,
                WarshipsGenericCollection<DrawingWarship, 
                DrawingObjectWarship>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (!_warshipStorages.ContainsKey(new WarshipsCollectionInfo(name,
                string.Empty)))
            {
                _warshipStorages.Add(new WarshipsCollectionInfo(name, 
                    string.Empty), new WarshipsGenericCollection
                    <DrawingWarship, DrawingObjectWarship>
                    (_pictureWidth, _pictureHeight));
            }
            else
            {
                MessageBox.Show("This set already exists");
            }
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (!_warshipStorages.ContainsKey(new WarshipsCollectionInfo(name,
                string.Empty)))
                return;
            _warshipStorages.Remove(new WarshipsCollectionInfo(name, string.Empty));
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public WarshipsGenericCollection<DrawingWarship, DrawingObjectWarship>
            ?this[string ind]
        {
            get
            {
                WarshipsCollectionInfo indObj = new WarshipsCollectionInfo(ind,
                    string.Empty);
                if (_warshipStorages.ContainsKey(indObj))
                    return _warshipStorages[indObj];
                return null;
            }
        }
        /// <summary>
        /// Сохранение информации по кораблям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<WarshipsCollectionInfo,
                WarshipsGenericCollection
                <DrawingWarship, DrawingObjectWarship>> 
                record in _warshipStorages)
            {
                StringBuilder records = new();
                foreach (DrawingWarship? elem in record.Value.GetWarships)
                {
                    records.Append(
                        $"{elem?.GetDataForSave(_separatorForObject)}" +
                        $"{_separatorRecords}");
                }
                data.AppendLine($"{record.Key.Name}{_separatorForKeyValue}" +
                    $"{records}");
            }
            if (data.Length == 0)
            {
                throw new Exception("Invalid operation, no data to save");
            }
            using FileStream fs = new(filename, FileMode.Create);
            byte[] info = new UTF8Encoding(true).GetBytes
                ($"WarshipStorage{Environment.NewLine}{data}");
            fs.Write(info, 0, info.Length);
            return;
        }
        /// <summary>
        /// Загрузка информации по кораблям в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new Exception("File not found");
            }
            string bufferTextFromFile = "";
            using (FileStream fs = new(filename, FileMode.Open))
            {
                byte[] b = new byte[fs.Length];
                UTF8Encoding temp = new(true);
                while (fs.Read(b, 0, b.Length) > 0)
                {
                    bufferTextFromFile += temp.GetString(b);
                }
            }
            var strs = bufferTextFromFile.Split(new char[] { '\n', '\r' },
                StringSplitOptions.RemoveEmptyEntries);
            if (strs == null || strs.Length == 0)
            {
                throw new Exception("No data to download");
            }
            if (!strs[0].StartsWith("WarshipStorage"))
            {
                throw new Exception("Invalid data format");
            }
            _warshipStorages.Clear();
            foreach (string data in strs)
            {
                string[] record = data.Split(_separatorForKeyValue, 
                    StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 2)
                {
                    continue;
                }
                WarshipsGenericCollection<DrawingWarship, DrawingObjectWarship>
                collection = new(_pictureWidth, _pictureHeight);
                string[] set = record[1].Split(_separatorRecords, 
                    StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    DrawingWarship? warship =
                        elem?.CreateDrawingWarship(_separatorForObject, 
                        _pictureWidth, _pictureHeight);
                    if (warship != null)
                    {
                        if (!(collection + warship))
                        {
                            throw new Exception("Error adding to collection");
                        }
                    }
                }
                _warshipStorages.Add(new WarshipsCollectionInfo(record[0], 
                    string.Empty), collection);
            }
        }
    }
}